@@ -1,16 +1,10 @@
+from ._utils import _developer_utils
+from ._Forecaster_parent import ForecastError
+from .Forecaster import Forecaster
 import pandas as pd
 import numpy as np
 import warnings
-from scalecast.Forecaster import (
-    Forecaster,
-    ForecastError,
-    rmse,
-    mape,
-    mae,
-    r2,
-    _return_na_if_len_zero,
-    _convert_m,
-)
+import logging
 
 class SeriesTransformer:
     def __init__(self, f, deepcopy=True):
@@ -52,20 +46,18 @@ class SeriesTransformer:
         self.f.levely = list(transform_func(self.f.levely, **kwargs))
         return self.f
 
-    def Revert(self, revert_func, full=True, exclude_models = [], **kwargs):
+    def Revert(self, revert_func, exclude_models = [], **kwargs):
         """ Reverts the y attribute in the Forecaster object, along with all model results.
 
         Args:
             revert_func (function): The function that will be used to revert the values.
                 If using a user function, first argument must be the array to transform.
-            full (bool): Whether to revert all attributes, or just the level attributes.
-                If False, all non-level results will remain unchanged.
             exclude_models (list-like): Models to not revert. This is useful if you are transforming
                 and reverting an object over and over.
             **kwargs: Passed to the function passed to revert_func.
 
         Returns:
-            (Forecaster): a Forecaster object with the reverted attributes.
+            (Forecaster): A Forecaster object with the reverted attributes.
 
         >>> import math
         >>> from scalecast.Forecaster import Forecaster
@@ -80,12 +72,20 @@ class SeriesTransformer:
         >>> f = transformer.Revert(log10_revert)
         """
         self.f.levely = list(revert_func(self.f.levely, **kwargs))
-        self.f.y = pd.Series(revert_func(self.f.y, **kwargs)) if full else self.f.y
+        self.f.y = pd.Series(revert_func(self.f.y, **kwargs))
 
         for m, h in self.f.history.items():
             if m in exclude_models:
                 continue
             for k in ( 
+                "Forecast",
+                "TestSetPredictions",
+                "TestSetUpperCI",
+                "TestSetLowerCI",
+                "UpperCI",
+                "LowerCI",
+                "TestSetActuals",
+                "FittedVals",
                 "LevelForecast", 
                 "LevelTestSetPreds", 
                 "LevelFittedVals",
@@ -94,49 +94,30 @@ class SeriesTransformer:
                 "LevelUpperCI",
                 "LevelTSUpperCI",
             ):
-                h[k] = pd.Series(revert_func(h[k], **kwargs)).fillna(method='ffill').to_list()
+                try:
+                    h[k] = pd.Series(revert_func(h[k], **kwargs),dtype=float).fillna(method='ffill').to_list()
+                except KeyError:
+                    pass
             h['LevelY'] = self.f.levely
-
-            for i, preds in enumerate(('LevelTestSetPreds','LevelFittedVals')):
-                pred = h[preds]
-                act = self.f.levely[-len(pred) :]
-                if i == 0:
-                    h["LevelTestSetRMSE"] = _return_na_if_len_zero(act, pred, rmse)
-                    h["LevelTestSetMAPE"] = _return_na_if_len_zero(act, pred, mape)
-                    h["LevelTestSetMAE"] = _return_na_if_len_zero(act, pred, mae)
-                    h["LevelTestSetR2"] = _return_na_if_len_zero(act, pred, r2)
-                elif i == 1:
-                    h["LevelInSampleRMSE"] = rmse(act, pred)
-                    h["LevelInSampleMAPE"] = mape(act, pred)
-                    h["LevelInSampleMAE"] = mae(act, pred)
-                    h["LevelInSampleR2"] = r2(act, pred)
-
-            if full:
-                for k in (
-                    "Forecast",
-                    "TestSetPredictions",
-                    "TestSetUpperCI",
-                    "TestSetLowerCI",
-                    "UpperCI",
-                    "LowerCI",
-                    "TestSetActuals",
-                    "FittedVals",
-                ):
-                    h[k] = pd.Series(revert_func(h[k], **kwargs)).fillna(method='ffill').to_list()
-
-                for i, preds in enumerate(("TestSetPredictions","FittedVals")):
-                    pred = h[preds]
-                    act = self.f.levely[-len(pred) :]
-                    if i == 0:
-                        h["TestSetRMSE"] = _return_na_if_len_zero(act, pred, rmse)
-                        h["TestSetMAPE"] = _return_na_if_len_zero(act, pred, mape)
-                        h["TestSetMAE"] = _return_na_if_len_zero(act, pred, mae)
-                        h["TestSetR2"] = _return_na_if_len_zero(act, pred, r2)
-                    elif i == 1:
-                        h["InSampleRMSE"] = rmse(act, pred)
-                        h["InSampleMAPE"] = mape(act, pred)
-                        h["InSampleMAE"] = mae(act, pred)
-                        h["InSampleR2"] = r2(act, pred)
+            for met, func in self.f.metrics.items():
+                h['TestSet' + met.upper()] = _developer_utils._return_na_if_len_zero(
+                    self.f.levely[-len(h["TestSetPredictions"]) :], 
+                    h["TestSetPredictions"], 
+                    func,
+                )
+                h['LevelTestSet' + met.upper()] = _developer_utils._return_na_if_len_zero(
+                    self.f.levely[-len(h["LevelTestSetPreds"]) :], 
+                    h["LevelTestSetPreds"], 
+                    func,
+                )
+                h['InSample' + met.upper()] = func(
+                    self.f.levely[-len(h['FittedVals']) :], 
+                    h['FittedVals'], 
+                )
+                h['LevelInSample' + met.upper()] = func(
+                    self.f.levely[-len(h['LevelFittedVals']) :], 
+                    h['LevelFittedVals'],
+                )
 
         return self.f
 
@@ -191,7 +172,7 @@ class SeriesTransformer:
 
         if seasonal_lags > 0:
             if m == 'auto':
-                m = _convert_m(m,fmod.freq)
+                m = _developer_utils._convert_m(m,fmod.freq)
                 if m == 1:
                     warnings.warn(
                         f'cannot add seasonal lags automatically for the {fmod.freq} frequency. '
@@ -235,9 +216,11 @@ class SeriesTransformer:
 
         # i'm not 100% sure we need this and it does cause one thing to break so i'm doing this for now.
         try: 
-            self.f.typ_set(); 
+            self.f._typ_set(); 
         except: 
-            warnings.warn('type seting the Forecaster object did not work in the trend transform, continuing as is.')
+            logging.warning(
+                'Forecaster._typ_set() did not work after the detrend transformation, continuing as is.'
+            )
 
         self.detrend_model = ols_mod
         self.detrend_fvs = fvs
@@ -272,7 +255,7 @@ class SeriesTransformer:
             self.f.y = self.detrend_origy
             self.f.current_dates = self.detrend_origdates
         except AttributeError:
-            raise ForecastError('before reverting a trend, make sure DetrendTransform has already been called.')
+            raise ForecastError('Before reverting a trend, make sure DetrendTransform has already been called.')
 
         fvs = {
             "LevelTestSetPreds":self.detrend_fvs_test,
@@ -295,7 +278,10 @@ class SeriesTransformer:
             if m in exclude_models:
                 continue
             for k,v in fvs.items():
-                h[k] = [i + fvs for i, fvs in zip(h[k],v)]
+                try:
+                    h[k] = [i + fvs for i, fvs in zip(h[k],v)]
+                except KeyError:
+                    pass
 
         for a in (
             'detrend_origy',
@@ -330,7 +316,7 @@ class SeriesTransformer:
 
         Args:
             **kwargs: Passed to Transformer.Revert() - 
-                args `full` and `exclude_models` accepted here.
+                arg `exclude_models` accepted here.
 
         Returns:
             (Forecaster): A Forecaster object with the reverted attributes.
@@ -364,7 +350,7 @@ class SeriesTransformer:
 
         Args:
             **kwargs: Passed to Transformer.Revert() - 
-                args `full` and `exclude_models` accepted here.
+                arg `exclude_models` accepted here.
 
         Returns:
             (Forecaster): A Forecaster object with the reverted attributes.
@@ -414,7 +400,7 @@ class SeriesTransformer:
 
         Args:
             **kwargs: Passed to Transformer.Revert() - 
-                args `full` and `exclude_models` accepted here.
+                arg `exclude_models` accepted here.
 
         Returns:
             (Forecaster): A Forecaster object with the reverted attributes.
@@ -475,7 +461,7 @@ class SeriesTransformer:
 
         Args:
             **kwargs: {assed to Transformer.Revert() - 
-                args `full` and `exclude_models` accepted here.
+                arg `exclude_models` accepted here.
 
         Returns:
             (Forecaster): A Forecaster object with the reverted attributes.
@@ -501,12 +487,12 @@ class SeriesTransformer:
             raise ValueError("cannot revert a series that was never scaled.")
 
     def DiffTransform(self, m=1):
-        """ takes differences or seasonal differences in the Forecaster object's y attribute.
+        """ Takes differences or seasonal differences in the Forecaster object's y attribute.
         If using this transformation, call `Forecaster.add_diffed_terms()` and 
         `Forecaster.add_lagged_terms()` if you want to use those before calling this function.
         Call `Forecaster.add_ar_terms()` and `Forecaster.add_AR_terms()` after calling this function.
         Call twice with the same value of m to take second differences.
-        iI using this to take series differences, do not also use the native `Forecaster.diff()` function.
+        If using this to take series differences, do not also use the native `Forecaster.diff()` function.
 
         Args:
             m (int): Default 1. The seasonal difference to take.
@@ -548,15 +534,16 @@ class SeriesTransformer:
     def DiffRevert(self, m=1, exclude_models = []):
         """ Reverts the y attribute in the Forecaster object, along with all model results.
         Calling this makes so that AR values become unusable and have to be re-added to the object.
-        Unlike other revert functions, there is no option for full = False. 
         If using this to revert differences, you should not also use the native Forecaster.diff() function.
 
         Args:
             m (int): Default 1. The seasonal difference to revert.
                 1 will undifference once. 
                 12 will undifference seasonally 12 periods (monthly data).
-                Any int available. use the same values to revert as you used
+                Any int available. Use the same values to revert as you used
                 to transform the object originally.
+            exclude_models (list-like): Models to not revert. This is useful if you are transforming
+                and reverting an object over and over.
 
         Returns:
             (Forecaster): A Forecaster object with the reverted attributes.
@@ -614,16 +601,35 @@ class SeriesTransformer:
                 seasrevert(h['FittedVals'],self.f.y.values[-len(h['FittedVals'])-m:], m)
             )[m:]
             h['LevelFittedVals'] = h['FittedVals'][:]
-            for k in ("LevelUpperCI","UpperCI","LevelLowerCI","LowerCI"):
-                h[k] = list(seasrevert(h[k], self.f.y[-m:], m))[m:]
-            for k in ("TestSetUpperCI","LevelTSUpperCI","TestSetLowerCI","LevelTSLowerCI"):
-                h[k] = list(
-                    seasrevert(
-                        h[k],
-                        self.f.y.to_list()[-(m + self.f.test_length) : -self.f.test_length],
-                        m,
-                    )
-                )[m:]
+            if "LevelUpperCI" not in self.f.history[mod].keys(): # no cis evaluated
+                continue
+
+            # undifference cis
+            fcst = h["Forecast"]
+            test_preds = h["TestSetPredictions"]
+            test_actuals = h["TestSetActuals"]
+            test_resids = np.abs([p - a for p, a in zip(test_preds,test_actuals)])
+            ci_range = np.percentile(test_resids, 100 * self.f.cilevel)
+            self.f._set_cis(
+                "UpperCI",
+                "LowerCI",
+                "TestSetUpperCI",
+                "TestSetLowerCI",
+                m = mod,
+                ci_range = ci_range,
+                forecast = fcst,
+                tspreds = test_preds,
+            )
+            self.f._set_cis(
+                "LevelUpperCI",
+                "LevelLowerCI",
+                "LevelTSUpperCI",
+                "LevelTSLowerCI",
+                m = mod,
+                ci_range = ci_range,
+                forecast = fcst,
+                tspreds = test_preds,
+            )
         delattr(self, f"orig_y_{m}_{n}")
         delattr(self, f"orig_dates_{m}_{n}")
 
