@@ -1,24 +1,19 @@
+from ._Forecaster_parent import _tune_test_forecast
+from .Forecaster import Forecaster
+from .MVForecaster import MVForecaster
 import typing
 from typing import Dict, Union
-
 from ipywidgets import widgets
 from IPython.display import display, clear_output
-
-from scalecast.Forecaster import Forecaster, _determine_best_by_, _estimators_
-
 import matplotlib.pyplot as plt
 import seaborn as sns
 
-from tqdm.notebook import tqdm as log_progress
-
-
 def results_vis(
-    f_dict: Dict[str, Forecaster],
+    f_dict: Dict[str,Forecaster],
     plot_type: str = "forecast",
-    print_attr: list = [],
     include_train: Union[bool, int] = True,
     figsize = (12,6),
-) -> None:
+):
     """ Visualize the forecast results from many different Forecaster objects leveraging Jupyter widgets.
 
     Args:
@@ -26,10 +21,6 @@ def results_vis(
             Works best if two or more models have been evaluated in each dictionary value.
         plot_type (str): One of {"forecast","test"}. Default "forecast".
             The type of results to visualize.
-        print_attr (list): Optional.
-            The attributes from history to print.
-            Passed to print_attr parameter when plot_type = 'forecast'.
-            Ignored when plot_type = 'test'.
         include_train (bool or int): Optional.
             Whether to include the complete training set in the plot or how many traning-set observations to include.
             Passed to include_train parameter when plot_type = 'test'.
@@ -51,7 +42,6 @@ def results_vis(
                 models=f"top_{mo_selection}",
                 order_by=me_selection,
                 level=lv_selection,
-                print_attr=print_attr,
                 ci=ci_selection,
                 figsize=figsize,
             )
@@ -86,12 +76,14 @@ def results_vis(
     mo_dd = widgets.Dropdown(
         options=range(1, len(all_models) + 1), description="No. Models"
     )
-    lv_dd = widgets.Dropdown(options=[True, False], description="View Level")
+    lv_dd = widgets.Dropdown(options=[False, True], description="View Level")
     ci_dd = widgets.Dropdown(
-        options=[True, False], description="View Confidence Intervals"
+        options=[False, True], description="View Confidence Intervals"
     )
     me_dd = widgets.Dropdown(
-        options=sorted(_determine_best_by_), description="Order By"
+        options=sorted(f.determine_best_by), # f will be last object iterated through above
+        description="Order By", 
+        value = [m for m in f.determine_best_by if m.startswith('TestSet')][0],
     )
 
     # never changes
@@ -104,7 +96,12 @@ def results_vis(
     button.on_click(on_button_clicked)
 
 
-def results_vis_mv(f_dict, plot_type="forecast", include_train=True, figsize = (12,6)):
+def results_vis_mv(
+    f_dict: Dict[str,MVForecaster], 
+    plot_type="forecast", 
+    include_train=True, 
+    figsize = (12,6)
+):
     """ Visualize the forecast results from many different MVForecaster objects leveraging Jupyter widgets.
 
     Args:
@@ -173,9 +170,9 @@ def results_vis_mv(f_dict, plot_type="forecast", include_train=True, figsize = (
     se_se = widgets.SelectMultiple(
         options=series, description="Series", selected=series
     )
-    lv_dd = widgets.Dropdown(options=[True, False], description="View Level")
+    lv_dd = widgets.Dropdown(options=[False, True], description="View Level")
     ci_dd = widgets.Dropdown(
-        options=[True, False], description="View Confidence Intervals"
+        options=[False, True], description="View Confidence Intervals"
     )
 
     # never changes
@@ -194,21 +191,20 @@ def tune_test_forecast(
     cross_validate=False,
     dynamic_tuning=False,
     dynamic_testing=True,
-    probabilistic=False,
-    n_iter=20,
     summary_stats=False,
     feature_importance=False,
     fi_method="pfi",
     limit_grid_size=None,
     suffix=None,
+    error='raise',
     **cvkwargs,
 ):
     """ Tunes, tests, and forecasts a series of models with a progress bar through tqdm.
 
     Args:
-        f (Forecaster or MVForecaster): the object to run the models through.
+        f (Forecaster or MVForecaster): The object to run the models through.
         models (list-like):
-            Each element must be in _can_be_tuned_.
+            Each element must be in Forecaster.can_be_tuned.
         cross_validate (bool): Default False.
             Whether to tune the model with cross validation. 
             If False, uses the validation slice of data to tune.
@@ -220,12 +216,6 @@ def tune_test_forecast(
             If int, window evaluates over that many steps (2 for 2-step recurvie testing, 12 for 12-step, etc.).
             Setting this to False or 1 means faster performance, 
             but gives a less-good indication of how well the forecast will perform more than one period out.
-        probabilistic (bool, str, or list-like): Default False.
-            If bool, whether to use a probabilistic forecasting process to set confidence intervals for all models.
-            If str, the name of a single model to apply a probabilistic process to.
-            If list-like, a list of models to apply a probabilistic process to.
-        n_iter (int): Default 20.
-            How many iterations to use in probabilistic forecasting. Ignored if probabilistic = False.
         summary_stats (bool): Default False.
             Whether to save summary stats for the models that offer those.
             Does not work for `MVForecaster` objects.
@@ -240,43 +230,25 @@ def tune_test_forecast(
             See https://scalecast.readthedocs.io/en/latest/Forecaster/Forecaster.html#src.scalecast.Forecaster.Forecaster.limit_grid_size.
         suffix (str): Optional. A suffix to add to each model as it is evaluate to differentiate them when called
             later. If unspecified, each model can be called by its estimator name.
+        error (str): One of 'ignore','raise','warn'; default 'raise'.
+            What to do with the error if a given model fails.
+            'warn' prints a warning that the model could not be evaluated.
         **cvkwargs: Passed to the cross_validate() method.
 
     Returns:
         None
     """
-    if (
-        len([m for m in models if m not in [m for m in _estimators_ if m != "combo"]])
-        > 0
-    ):
-        raise ValueError(
-            "values passed to models must be list-like and in {}".format(
-                [m for m in _estimators_ if m != "combo"]
-            )
-        )
-    for m in log_progress(models):
-        m_prob = (
-            probabilistic if isinstance(probabilistic,bool) 
-            else m == probabilistic if isinstance(probabilistic,str) 
-            else m in probabilistic
-        )
-        call_me = m if suffix is None else m + suffix
-        f.set_estimator(m)
-        if limit_grid_size is not None:
-            f.ingest_grid(m)
-            f.limit_grid_size(limit_grid_size)
-        if cross_validate:
-            f.cross_validate(dynamic_tuning=dynamic_tuning, **cvkwargs)
-        else:
-            f.tune(dynamic_tuning=dynamic_tuning)
-        f.auto_forecast(
-            dynamic_testing=dynamic_testing,
-            call_me=call_me,
-            probabilistic=m_prob,
-            n_iter=n_iter,
-        )
-
-        if summary_stats:
-            f.save_summary_stats()
-        if feature_importance:
-            f.save_feature_importance(fi_method)
+    _tune_test_forecast(
+        f=f,
+        models=models,
+        cross_validate=cross_validate,
+        dynamic_tuning=dynamic_tuning,
+        dynamic_testing=dynamic_testing,
+        limit_grid_size=limit_grid_size,
+        suffix=suffix,
+        error=error,
+        summary_stats=summary_stats,
+        feature_importance=feature_importance,
+        fi_method=fi_method,
+        tqdm = True,
+    )
