@@ -1,16 +1,18 @@
-from scalecast import Forecaster
-from scalecast import MVForecaster
-from scalecast import SeriesTransformer
-from scalecast.auxmodels import auto_arima
-from scalecast import Pipeline
-import pandas_datareader as pdr
 import matplotlib.pyplot as plt
 import seaborn as sns
 import numpy as np
 import pandas as pd
 import warnings
+from sklearn.metrics import (
+    r2_score,
+    mean_squared_error,
+    mean_absolute_percentage_error,
+    mean_absolute_error,
+)
+from ._utils import _developer_utils
 
 class metrics:
+    @staticmethod
     def mape(a,f):
         """ Mean absolute percentage error (MAPE).
 
@@ -26,8 +28,13 @@ class metrics:
         >>> f = [1,2,3,4,6]
         >>> metrics.mape(a,f)
         """
-        return Forecaster.mape(a,f)
+        return (
+            np.nan if np.abs(a).min() == 0 
+            else mean_absolute_percentage_error(a, f)
+        )
 
+
+    @staticmethod
     def r2(a,f):
         """ R-squared (R2).
 
@@ -43,8 +50,9 @@ class metrics:
         >>> f = [1,2,3,4,6]
         >>> metrics.r2(a,f)
         """
-        return Forecaster.r2(a,f)
+        return r2_score(a, f)
 
+    @staticmethod
     def mse(a,f):
         """ Mean squared error (MSE).
 
@@ -60,8 +68,9 @@ class metrics:
         >>> f = [1,2,3,4,6]
         >>> metrics.mse(a,f)
         """
-        return Forecaster.rmse(a,f)**2
+        return mean_squared_error(a, f)
 
+    @staticmethod
     def rmse(a,f):
         """ Root mean squared error (RMSE).
 
@@ -77,8 +86,9 @@ class metrics:
         >>> f = [1,2,3,4,6]
         >>> metrics.rmse(a,f)
         """
-        return Forecaster.rmse(a,f)
+        return mean_squared_error(a, f) ** 0.5
 
+    @staticmethod
     def mae(a,f):
         """ Mean absolute error (MAE).
 
@@ -94,8 +104,9 @@ class metrics:
         >>> f = [1,2,3,4,6]
         >>> metrics.mae(a,f)
         """
-        return Forecaster.mae(a,f)
+        return mean_absolute_error(a, f)
 
+    @staticmethod
     def smape(a,f):
         """ Symmetric mean absolute percentage error (sMAPE).
         Uses the same definition as used in the M4 competition.
@@ -125,6 +136,7 @@ class metrics:
             )
         )
 
+    @staticmethod
     def mase(a,f,obs,m):
         """ Mean absolute scaled error (MASE).
         Uses the same definition as used in the M4 competition.
@@ -157,11 +169,12 @@ class metrics:
         )
         return avger * (num / (davger * denom))
 
+    @staticmethod
     def msis(a,uf,lf,obs,m,alpha=0.05):
         """ Mean scaled interval score (MSIS) for evaluating confidence intervals.
         Uses the same definition as used in the M4 competition.
         Lower values are better.
-        see https://ideas.repec.org/a/eee/intfor/v36y2020i1p54-74.html.
+        See https://ideas.repec.org/a/eee/intfor/v36y2020i1p54-74.html.
 
         Args:
             a (list-like): The actuals over the forecast horizon.
@@ -196,91 +209,9 @@ class metrics:
         denom = np.sum(np.abs(pd.Series(obs).diff(m).values[m:]))
         return avger * (np.sum(num1 + num2) / (davger * denom))
 
-def pdr_load(
-    sym,
-    start=None,
-    end=None,
-    src='fred',
-    require_future_dates=True,
-    future_dates=None,
-    integrate=False,
-    diff=False,
-    MVForecaster_kwargs={},
-    **kwargs
-):
-    """ Gets data using `pandas_datareader.DataReader()` and loads the series into a Forecaster or MVForecaster object.
-    This functions works well when the src arg is its default ('fred'), but there are some issues with other sources.
-
-    Args:
-        sym (str or list-like): The name of the series to extract.
-            If str (one series), returns a Forecaster object.
-            If a collection, returns an MVForecaster object. 
-            Series of higher frequencies will having missing values filled using a forward fill.
-        start (str or datetime): The start date to extract data.
-        end (str or datetime): The end date to extract data.
-        src (str): The source of the API pull.
-            supported values: 'fred', 'yahoo', 'alphavantage', 'enigma', 
-            'famafrench','moex', 'quandl', 'stooq', 'tiingo'.
-        require_future_dates (bool): Default True.
-            If False, none of the models from the resulting Forecaster object 
-            will forecast into future periods by default.
-            If True, all models will forecast into future periods, 
-            unless run with test_only = True, and when adding regressors, they will automatically
-            be added into future periods.
-            Always set to True if sym is list-like (MVForecaster doesn't do test only).
-        future_dates (int): Optional. The future dates to add to the model upon initialization.
-            If not added when object is initialized, can be added later.
-        integrate (bool): Default False. Whether to take first differences in extraced data if it is found to be non-stationary.
-        diff (bool): Default False. Whether to take first differences in extracted data 
-            without running the ADF test to check stationarity.
-        MVForecaster_kwargs (dict): Default {}. If sym is list-like, 
-            these arguments will be passed to the `MVForecaster()` init function.
-            If 'names' is not found in the dict, names are automatically added so that the
-            MVForecaster keeps the names of the extracted symbols.
-            To keep no names, pass `MVForecaster_kwargs = {'names':None,...}`.
-        **kwargs: Passed to pdr.DataReader() function. 
-            See https://pandas-datareader.readthedocs.io/en/latest/remote_data.html.
-
-    Returns:
-        (Forecaster or MVForecaster): An object with the dates and y-values loaded.
-    """
-    df = pdr.DataReader(sym,data_source=src,start=start,end=end,**kwargs)
-    if isinstance(sym,str):
-        f = Forecaster.Forecaster(
-            y=df[sym],
-            current_dates=df.index,
-            require_future_dates=require_future_dates,
-            future_dates = future_dates,
-        )
-        if integrate:
-            f.integrate()
-        elif diff:
-            f.diff()
-        return f
-    else:
-        fs = []
-        for s in sym:
-            df[s].fillna(method='ffill',inplace=True)
-            f = Forecaster.Forecaster(
-                y = df[s],
-                current_dates=df.index,
-                future_dates = future_dates,
-            )
-            if integrate:
-                f.integrate()
-            elif diff:
-                f.diff()
-            fs.append(f)
-        if 'names' not in MVForecaster_kwargs:
-            MVForecaster_kwargs['names'] = sym
-        return MVForecaster.MVForecaster(
-            *fs,
-            **MVForecaster_kwargs,
-        )
-
 def plot_reduction_errors(f, ax = None, figsize=(12,6)):
     """ Plots the resulting error/accuracy of a Forecaster object where `reduce_Xvars()` method has been called
-    with method = 'pfi'.
+    with method = 'pfi' or method = 'shap'.
     
     Args:
         f (Forecaster): An object that has called the `reduce_Xvars()` method with method = 'pfi'.
@@ -289,6 +220,26 @@ def plot_reduction_errors(f, ax = None, figsize=(12,6)):
         
     Returns:
         (Axis) The figure's axis.
+
+    >>> from scalecast.Forecaster import Forecaster
+    >>> from scalecast.util import plot_reduction_errors
+    >>> import matplotlib.pyplot as plt
+    >>> import seaborn as sns
+    >>> import pandas as pd
+    >>> import pandas_datareader as pdr
+    >>> 
+    >>> df = pdr.get_data_fred('HOUSTNSA',start='1900-01-01',end='2021-06-01')
+    >>> f = Forecaster(y=df['HOUSTNSA'],current_dates=df.index)
+    >>> f.set_test_length(.2)
+    >>> f.generate_future_dates(24)
+    >>> f.add_ar_terms(24)
+    >>> f.integrate(critical_pval=.01)
+    >>> f.add_seasonal_regressors('month',raw=False,sincos=True,dummy=True)
+    >>> f.add_seasonal_regressors('year')
+    >>> f.add_time_trend()
+    >>> f.reduce_Xvars(method='pfi')
+    >>> plot_reduction_errors(f)
+    >>> plt.show()
     """
     dropped = f.pfi_dropped_vars
     errors = f.pfi_error_values
@@ -301,6 +252,149 @@ def plot_reduction_errors(f, ax = None, figsize=(12,6)):
     plt.ylabel("error")
     return ax
 
+def backtest_metrics(
+    backtest_results: list,
+    mets = ['rmse'],
+    mase = False, 
+    msis = False,
+    msis_alpha = 0.05,
+    m = 1, 
+    names = None,
+) -> pd.DataFrame:
+    """ Ingests the results output from `Pipeline.backtest()` and converts results to metrics.
+    
+    Args:
+        backtest_results (list): The output returned from `Pipeline.backtest()` or `MVPipeline.backtest()`.
+        mets (list): Default ['rmse']. A list of metrics to calculate.
+            If the element is str type, must be taked from the `util.metrics` class 
+            where the only two accepted arguments are a and f.
+            If the element in the list is callable, must be a function that only accepts two arguments and returns a float.
+        mase (bool): Default False.
+            Whether to also calculate mase. Must specify seasonality in m.
+        msis (bool): Default False.
+            Whether to also calculate msis. Must specify seasonality in m. This will fail if confidence intervals were not evaluated.
+        msis_alpha (float): Default 0.05. The level that confidence intervals were evaluated at. Ignored if msis is False.
+        m (int): Default 1. The number of steps that count one seasonal cycle. Ignored if both of msis and mase is False.
+        names (list): Optional. The names to assign each passed series. Ignored if there is only one passed series.
+    
+    Returns:
+        (DataFrame): The metrics dataframe that gives info about each backtested series, model, and selected metric.
+
+    >>> f1, f2, f3 = pipeline.fit_predict(f1,f2,f3)
+    >>> backtest_results = pipeline.backtest(f1,f2,f3,n_iter=2,jump_back=12)
+    >>> backtest_mets = backtest_metrics(
+    >>>     backtest_results,
+    >>>     mets = ['rmse','smape','r2','mae'],
+    >>>     names=['UTUR','UNRATE','SAHMREALTIME'],
+    >>>     mase = True,
+    >>>     msis = True,
+    >>>     m = 12,
+    >>> )
+    """
+    m_ = m
+    labels = names if names is not None else [f'Series{i}' for i in range(len(backtest_results))]
+    res_dict = {k:None for k in labels}
+    mets_str = [m if isinstance(m,str) else m.__name__ for m in mets]
+    for h, s in enumerate(backtest_results):
+        for i, m in enumerate(s):
+            if i == 0:
+                a_df = s[m][[c for c in s[m] if c.endswith('Vals')]]
+                results = pd.DataFrame(
+                    columns = [f'Iter{i}' for i in range(a_df.shape[1])],
+                    index = pd.MultiIndex.from_product(
+                            [
+                                [m for m in backtest_results[0] if m not in ('Actuals','Obs')],
+                                mets_str + ([] if not mase else ['mase']) + ([] if not msis else ['msis']),
+                            ],
+                            names = ['Model','Metric']
+                    ),
+                )
+                a_df.columns = results.columns.to_list()
+            elif i == 1:
+                ob_df = s[m]
+            else:
+                f_df = s[m][[c for c in s[m] if c.endswith('Fcst')]]
+                f_df.columns = results.columns.to_list()
+                for met in mets_str:
+                    for c in results:
+                        results.loc[(m,met),c] = (
+                            getattr(metrics,met)(
+                                a = a_df[c],
+                                f = f_df[c],
+                            ) if isinstance(met,str) 
+                            else met(
+                                a_df[c],
+                                f_df[c],
+                            )
+                        )
+                if mase:
+                    for c in results:
+                        results.loc[(m,'mase'),c] = getattr(metrics,'mase')(
+                            a = a_df[c],
+                            f = f_df[c],
+                            obs = ob_df[c].dropna(),
+                            m = m_,
+                        )
+                if msis:
+                    up_df = s[m][[c for c in s[m] if c.endswith('Upper')]]
+                    lo_df = s[m][[c for c in s[m] if c.endswith('Lower')]]
+                    up_df.columns = f_df.columns.to_list()
+                    lo_df.columns = f_df.columns.to_list()
+                    for c in results:
+                        results.loc[(m,'msis'),c] = getattr(metrics,'msis')(
+                            a = a_df[c],
+                            uf = up_df[c],
+                            lf = lo_df[c],
+                            obs = ob_df[c].dropna(),
+                            m = m_,
+                            alpha = msis_alpha,
+                        )
+        res_dict[labels[h]] = results
+
+    if len(res_dict) == 1:
+        results_df = res_dict[labels[0]]
+    else:
+        res_list = []
+        for k, df in res_dict.items():
+            df['Series'] = k
+            df = df.reset_index().set_index(['Series','Model','Metric'])
+            res_list.append(df)
+        results_df = pd.concat(res_list)
+
+    results_df['Average'] = results_df.mean(axis=1)
+    return results_df
+
+def _backtest_plot(
+    backtest_results,
+    models = None,
+    series = None,
+    names = None,
+    ci = False,
+    ax=None,
+    figsize=(12,6),
+):
+    """ (Coming soon). Plots the results from a backtested pipeline. If all default arguments are passed, the resulting plot can
+    be messy, so it's recommended to limit some of the output by specifying arguments.
+
+    Args:
+        backtest_results (list): The output returned from `Pipeline.backtest()` or `MVPipeline.backtest()`.
+        models (list): Which models to plot results for. By default, all models will be plotted.
+        series (list): Which series to plot results for. This argument expects a list with an index position.
+            Ex: [0,2,3] will plot series in the first, third, and fourth index positions of backtest_results.
+        names (list): Optional. The names to give to the corresponding series. Must be the same length as series.
+            If not specified, the series will be named 'Series1' - 'SeriesN'.
+        ci (bool): Default False. Whether to plot confidence intervals. This will raise a warning if confidence intervals are not found.
+        ax (Axis): Optional. A pre-existing axis to write the figure to.
+        figsize (tuple): Default (12,6). The size of the resulting figure. Ignored when ax is specified.
+    
+    Returns:
+        (Axis): The figure's axis.
+
+    >>>
+    >>>
+    """
+    if ax is None:
+        _, ax = plt.subplots(figsize=figsize)
 
 def break_mv_forecaster(mvf):
     """ Breaks apart an MVForecaster object and returns as many Foreaster objects as series loaded into the object.
@@ -309,9 +403,14 @@ def break_mv_forecaster(mvf):
         mvf (MVForecaster): The object to break apart.
 
     Returns:
-        (tuple): A sequence of at least two Forecaster objects
+        (tuple[Forecaster]): A sequence of at least two Forecaster objects
+    
+    >>> from scalecast.MVForecaster import MVForecaster
+    >>> from scalecast.util import break_mv_forecaster
+    >>> 
+    >>> f1, f2 = break_mv_forecaster(mvf)
     """
-
+    from .Forecaster import Forecaster
     def convert_mv_hist(f, mvhist: dict, series_num: int):
         hist = {}
         for k, v in mvhist.items():
@@ -325,7 +424,7 @@ def break_mv_forecaster(mvf):
                     try:
                         hist[k][k2] = list(v2.values())[series_num]
                     except IndexError:
-                        hist[k][k2] = np.nan
+                        hist[k][k2] = []
             hist[k]["TestOnly"] = False
             hist[k]["LevelY"] = f.levely
         return hist
@@ -339,7 +438,7 @@ def break_mv_forecaster(mvf):
     )
     to_return = []
     for s in range(mvf1.n_series):
-        f = Forecaster.Forecaster(
+        f = Forecaster(
             y=getattr(mvf1, f"series{s+1}")["y"].values[-set_len:],
             current_dates=mvf1.current_dates.values[-set_len:],
             integration=getattr(mvf1, f"series{s+1}")["integration"],
@@ -350,6 +449,8 @@ def break_mv_forecaster(mvf):
             future_xreg={k:v.copy() for k,v in mvf1.future_xreg.items()},
             test_length=mvf1.test_length,
             validation_length=mvf1.validation_length,
+            cis = mvf1.cis,
+            cilevel = mvf1.cilevel,
         )
         f.history = convert_mv_hist(f, mvf1.history, s)
         to_return.append(f)
@@ -368,6 +469,21 @@ def find_optimal_lag_order(mvf,train_only=False,**kwargs):
 
     Returns:
         (LagOrderResults): Lag selections.
+    
+    >>> from scalecast.Forecaster import Forecaster
+    >>> from scalecast.MVForecaster import MVForecaster
+    >>> from scalecast.util import find_optimal_lag_order
+    >>> import pandas_datareader as pdr
+    >>>
+    >>> s1 = pdr.get_data_fred('UTUR',start='2000-01-01',end='2022-01-01')
+    >>> s2 = pdr.get_data_fred('UNRATE',start='2000-01-01',end='2022-01-01')
+    >>>
+    >>> f1 = Forecaster(y=s1['UTUR'],current_dates=s1.index)
+    >>> f2 = Forecaster(y=s2['UNRATE'],current_dates=s2.index)
+    >>>
+    >>> mvf = MVForecaster(f1,f2,names=['UTUR','UNRATE'])
+    >>> lag_order_res = find_optimal_lag_order(mvf,train_only=True)
+    >>> lag_order_aic = lag_order_res.aic # picks the best lag order according to aic
     """
     from statsmodels.tsa.vector_ar.var_model import VAR
     data = np.array(
@@ -401,6 +517,22 @@ def find_optimal_coint_rank(mvf,det_order,k_ar_diff,train_only=False,**kwargs):
 
     Returns:
         (CointRankResults): Object containing the cointegration rank suggested by the test and allowing a summary to be printed.
+
+    >>> from scalecast.Forecaster import Forecaster
+    >>> from scalecast.MVForecaster import MVForecaster
+    >>> from scalecast.util import find_optimal_coint_rank
+    >>> import pandas_datareader as pdr
+    >>>
+    >>> s1 = pdr.get_data_fred('UTUR',start='2000-01-01',end='2022-01-01')
+    >>> s2 = pdr.get_data_fred('UNRATE',start='2000-01-01',end='2022-01-01')
+    >>>
+    >>> f1 = Forecaster(y=s1['UTUR'],current_dates=s1.index)
+    >>> f2 = Forecaster(y=s2['UNRATE'],current_dates=s2.index)
+    >>>
+    >>> mvf = MVForecaster(f1,f2,names=['UTUR','UNRATE'])
+    >>> coint_res = find_optimal_coint_rank(mvf,det_order=-1,k_ar_diff=8,train_only=True)
+    >>> print(coint_res) # prints a report
+    >>> rank = coint_res.rank # best rank
     """
     from statsmodels.tsa.vector_ar.vecm import select_coint_rank
     data = np.array(
@@ -430,26 +562,71 @@ def find_statistical_transformation(
 
     Args:
         f (Forecaster): The object that stores the series to test.
-        goal (list-like): One or multiple of 'stationary', 'seasonally_adj'. Other options may be coming in the future.
+        goal (list-like): Default ['stationary']. One or multiple of 'stationary', 'seasonally_adj'. 
+            Other options may be coming in the future.
             If more than one goal is passed, will try to satisfy all goals in the order passed.
             For stationary: uses an Augmented Dickey-Fuller test to determine if the series is stationary.
             If not stationary, returns a diff transformation and log transformation if log is True.
             For seasonall_adj: uses seasonal auto_arima to find the optimal seasonal diff.
         train_only (bool): Default False. Whether to use train set only in all statistical tests.
-        log (bool): Default True. Whether to log and diff the series if it is found to be non-stationary or just diff.
+        log (bool): Default True. Whether to log and difference the series if it is found to be non-stationary or just difference.
+            This will set itself to False if the lowest observed series value is 0 or lower.
         critical_pval (float): Default 0.05. The cutoff p-value to use to determine statistical signficance in the 
             Augmented Dickey-Fuller test and to run the auto_arima selection (substitutes for `alpha` arg).
         m (str or int): Default 'auto': The number of observations that counts one seasonal step.
             When 'auto', uses the M4 competition values: 
             for Hourly: 24, Monthly: 12, Quarterly: 4. everything else gets 1 (no seasonality assumed)
             so pass your own values for other frequencies.
-        adf_kwargs (dict): Default {}. Keyword args to pass to the Augmented Dickey-Fuller test function. 
+        adf_kwargs (dict): Default {}. Keyword args to pass to the Augmented Dickey-Fuller test function.
+            See the `maxlag`, `regression`, and `autolag` arguments from
+            https://www.statsmodels.org/dev/generated/statsmodels.tsa.stattools.adfuller.html.
         **kwargs: Passed to the auto_arima() function when searching for optimal seasonal diff.
 
     Returns:
         (Transformer, Reverter): A `Transformer` object with the identified transforming functions and
         the `Reverter` object with the `Transformer` counterpart functions.
+
+    >>> from scalecast.Forecaster import Forecaster
+    >>> from scaleast.Pipeline import Pipeline, Transformer, Reverter
+    >>> from scalecast.util import find_statistical_transformation
+    >>> import pandas_datareader as pdr
+    >>> 
+    >>> def forecaster(f):
+    >>>     f.add_covid19_regressor()
+    >>>     f.auto_Xvar_select(cross_validate=True)
+    >>>     f.set_estimator('mlr')
+    >>>     f.manual_forecast()
+    >>> df = pdr.get_data_fred(
+    >>>     'HOUSTNSA',
+    >>>     start='1959-01-01',
+    >>>     end='2022-08-01'
+    >>> )
+    >>> f = Forecaster(
+    >>>     y=df['HOUSTNSA'],
+    >>>     current_dates=df.index,
+    >>>     future_dates=24,
+    >>>     test_length = .2,
+    >>> )
+    >>> f.set_validation_length(24)
+    >>> transformer, reverter = find_statistical_transformation(
+    >>>     f,
+    >>>     goal=['stationary','seasonally_adj'],
+    >>>     train_only=True,
+    >>>     critical_pval = .01,
+    >>> )
+    >>> print(reverter) # see what transformers and reverters were chosen
+    >>> pipeline = Pipeline(
+    >>>   steps = [
+    >>>       ('Transform',transformer),
+    >>>       ('Forecast',forecaster),
+    >>>       ('Revert',reverter),
+    >>>   ],
+    >>> )
+    >>> f = pipeline.fit_predict(f)
     """
+    from . import SeriesTransformer
+    from . import Pipeline
+    from .auxmodels import auto_arima
     def make_stationary(f,train_only,critical_pval,log,adf_kwargs,**kwargs):
         transformers = []
         stationary = f.adf_test(
@@ -471,15 +648,16 @@ def find_statistical_transformation(
 
         auto_arima(f,m=m,seasonal=True,alpha=critical_pval,**kwargs)
         I = f.auto_arima_params['seasonal_order'][1]
-        for i in range(1,I+1):
-            transformers += [('DiffTransform',m)]
+        if I > 0:
+            for i in range(1,I+1):
+                transformers += [('DiffTransform',m)]
 
         return transformers
 
     f = f.deepcopy()
     transformer = SeriesTransformer.SeriesTransformer(f)
 
-    m = Forecaster._convert_m(m,f.freq)
+    m = _developer_utils._convert_m(m,f.freq)
 
     possible_args = {
         'stationary':make_stationary,
@@ -487,7 +665,7 @@ def find_statistical_transformation(
     }
     bad_args = [g for g in goal if g not in possible_args]
     if len(bad_args) > 0:
-        raise ValueError(f'values passed to goal arg cannot be used: {bad_args}')
+        raise ValueError(f'Values passed to goal arg cannot be used: {bad_args}.')
 
     transformers = []
     reverters = []
@@ -514,10 +692,11 @@ def find_statistical_transformation(
     )
     return final_transformer, final_reverter
 
+@_developer_utils.log_warnings
 def find_optimal_transformation(
     f,
     estimator='mlr',
-    monitor='TestSetRMSE',
+    monitor=None,
     lags='auto',
     try_order = ['detrend','boxcox','first_diff','first_seasonal_diff','scale'],
     boxcox_lambdas = [-0.5,0,0.5],
@@ -529,15 +708,17 @@ def find_optimal_transformation(
 ):
     """ Finds a set of transformations based on what maximizes forecast accuracy on some out-of-sample metric.
     Works by comparing each transformation individually and stacking the set of transformations that leads to the best
-    performance. The estimator only uses series lags as inputs.
+    performance. The estimator only uses series lags as inputs. When an attempted transformation fails, a warning is logged.
 
     Args:
         f (Forecaster): The Forecaster object that contains the series that will be transformed.
         estimator (str): One of _estimators_. Default 'mlr'. The estimator to use to choose the best 
             transformations with.
-        monitor (str): One of _determine_best_by_ except 'ValidationMetricValue'. Default 'TestSetRMSE'.
+        monitor (str): One of Forecaster.determine_best_by except 'ValidationMetricValue'. 
+            Default is 'TestSet' + first metric in Forecaster.metrics 
+            ('TestSetRMSE' is the default if no metric info has been manually specified).
             Because 'ValidationMetricValue' changes in scale based on the transformation taken, 
-            the metrics to monitor are limited to level in-sample and level out-of-sample metrics.
+            the metrics to monitor are limited to in-sample and test-set metrics.
             'TestSetRMSE' and 'LevelTestSetRMSE' are the same in this function, same with all level and non-level counterparts.
         lags (str or int): Default 'auto'. The number of lags that will be used as inputs for the estimator.
             If 'auto', uses the value passed or assigned to m (one seasonal cycle). 
@@ -557,7 +738,7 @@ def find_optimal_transformation(
             Only up to one scaler will be selected.
             Must exist a `SeriesTranformer.{scale_type}Transform()` function for this to work.
         scale_on_train_only (bool): Default False. Whether to fit the scaler on the training set only.
-        m (str, int, list[int]): Default 'auto': the number of observations that counts one seasonal step.
+        m (str, int, list[int]): Default 'auto'. The number of observations that counts one seasonal step.
             When 'auto', uses the M4 competition values: 
             for Hourly: 24, Monthly: 12, Quarterly: 4. everything else gets 1 (no seasonality assumed)
             so pass your own values for other frequencies. If m == 1, no first seasonal difference will be tried.
@@ -568,7 +749,42 @@ def find_optimal_transformation(
     Returns:
         (Transformer, Reverter): A `Transformer` object with the identified transforming functions and
         the `Reverter` object with the `Transformer` counterpart functions.
+
+    >>> from scalecast.Forecaster import Forecaster
+    >>> from scaleast.Pipeline import Pipeline, Transformer, Reverter
+    >>> from scalecast.util import find_optimal_transformation
+    >>> import pandas_datareader as pdr
+    >>> 
+    >>> def forecaster(f):
+    >>>     f.add_covid19_regressor()
+    >>>     f.auto_Xvar_select(cross_validate=True)
+    >>>     f.set_estimator('mlr')
+    >>>     f.manual_forecast()
+    >>> df = pdr.get_data_fred(
+    >>>     'HOUSTNSA',
+    >>>     start='1959-01-01',
+    >>>     end='2022-08-01'
+    >>> )
+    >>> f = Forecaster(
+    >>>     y=df['HOUSTNSA'],
+    >>>     current_dates=df.index,
+    >>>     future_dates=24,
+    >>>     test_length = .2, # this will be monitored for performance
+    >>> )
+    >>> f.set_validation_length(24)
+    >>> transformer, reverter = find_optimal_transformation(f)
+    >>> print(reverter) # see what transformers and reverters were chosen
+    >>> pipeline = Pipeline(
+    >>>   steps = [
+    >>>       ('Transform',transformer),
+    >>>       ('Forecast',forecaster),
+    >>>       ('Revert',reverter),
+    >>>   ],
+    >>> )
+    >>> f = pipeline.fit_predict(f)
     """
+    from . import Pipeline
+    from ._Forecaster_parent import ForecastError
     def forecaster(f):
         f.add_ar_terms(lags)
         f.set_estimator(estimator)
@@ -586,21 +802,29 @@ def find_optimal_transformation(
         )
         return pipeline.fit_predict(f)
 
+    if f.test_length == 0:
+        raise ForecastError(
+            'find_optimal_transformation() only works if a test length' 
+            ' above 0 is specified in the Forecaster object.'
+            ' Try calling the set_test_length(...) method.'
+        )
+
     f = f.deepcopy()
     f.drop_all_Xvars()
     f.history = {}
 
-    m = Forecaster._convert_m(m,f.freq)
+    m = _developer_utils._convert_m(m,f.freq)
     lags = m if lags == 'auto' and not hasattr(m,'__len__') else m[1] if lags == 'auto' else lags
     forecaster(f)
     
+    monitor = 'TestSet' + list(f.metrics.keys())[0].upper() if monitor is None else monitor
     level_met = f.export('model_summaries')[monitor].values[0]
     level_met = -level_met if monitor.endswith('R2') else level_met
 
     final_transformer = []
     final_reverter = []
 
-    exception_types = (IndexError,AttributeError,ValueError,ZeroDivisionError,Forecaster.ForecastError) # errors to pass over
+    exception_types = (IndexError,AttributeError,ValueError,ZeroDivisionError,ForecastError) # errors to pass over
 
     for tr in try_order:
         if tr == 'boxcox':
